(function() {
	'use strict';
	ObjC.import('AppKit');

	String.prototype.compareTo = function(other) {
		return this < other ? -1 : this > other ? 1 : 0;
	}

	var PlistUtils = (function() {
		function readTextFile(strPath) {
			var error;
			var str = ObjC.unwrap(
				$.NSString.stringWithContentsOfFileEncodingError(
					$(strPath).stringByStandardizingPath,
					$.NSUTF8StringEncoding,
					error
				)
			);
			if (error)
				throw Error('Could not read file "' + strPath + '"');
			return str;
		};

		return {
			convertPlistPartToString: function(plistPart) {
				var data = $.NSPropertyListSerialization.dataWithPropertyListFormatOptionsError(
					$(plistPart), $.NSPropertyListXMLFormat_v1_0, 0, null);
				var nsstring = $.NSString.alloc.initWithDataEncoding(data, $.NSUTF8StringEncoding);
				return $(nsstring).js;
			},

			convertStringToPlist: function(str) {
				return ObjC.deepUnwrap(
					$.NSPropertyListSerialization.propertyListWithDataOptionsFormatError(
						$(str).dataUsingEncoding($.NSUTF8StringEncoding), 0, 0, null));
			},

			readPlistArrayTextFile: function(strPath) {
				// var str = readTextFile(strPath);
				// return this.convertStringToPlist(str);
				var strFullPath = $(strPath).stringByStandardizingPath;
				return ObjC.deepUnwrap($.NSArray.arrayWithContentsOfFile(strFullPath));
			},

			readPlistBinaryFile: function(path) {
				var data = $.NSData.dataWithContentsOfFile(path);
				return ObjC.deepUnwrap(
					$.NSPropertyListSerialization.propertyListWithDataOptionsFormatError(
						data, $.NSPropertyListBinaryFormat_v1_0, 0, null));
			},

			readPlistDictionaryTextFile: function(strPath) {
				var strFullPath = $(strPath).stringByStandardizingPath;
				return ObjC.deepUnwrap($.NSDictionary.dictionaryWithContentsOfFile(strFullPath));
			},

			writePlistTextFile: function(plist, strPath) {
				var str = this.convertPlistPartToString(plist);
				$(str).writeToFileAtomically($(strPath).stringByStandardizingPath, true);
			}
		};
	})();

	var KMEngine = (function() {
		var _engineApp = undefined;

		return {
			convertStringToPlist: function(str) {
				return ObjC.deepUnwrap(
					$.NSPropertyListSerialization.propertyListWithDataOptionsFormatError(
						$(str).dataUsingEncoding($.NSUTF8StringEncoding), 0, 0, null));
			},

			getEngineAppName: function() {
				return "Keyboard Maestro Engine";
			},

			getEngineApp: function() {
				if (!_engineApp)
					_engineApp = Application(this.getEngineAppName());
				return _engineApp;
			},

			getMacros: function(binary) {
				return this.getEngineApp().getmacros({
					asstring: !binary
				});
			},

			getMacrosAsPlist: function() {
				return this.convertStringToPlist(this.getMacros(false));
			},

			getFlattenedMacrosAndGroups: function() {
				var plist = this.getMacrosAsPlist();
				var result = [];
				plist.forEach(function(group) {
					if (!group.macros)
						clontinue;

					group.macros.forEach(function(macro) {
						result.push({
							macroName: macro.name,
							macroUID: macro.uid,
							groupName: group.name,
							groupUID: group.uid
						})
					});
				});
				return result;
			},

			getVariable: function(name, required) {
				var result = this.getEngineApp().getvariable(name);
				if (!result && required)
					throw Error("Variable '" + name + "' is empty");
				return result;
			},

			setVariable: function(name, value) {
				this.getEngineApp().setvariable(name, {
					to: value
				});
			},
		};

	})();

	function run() {
		var _result = KMEngine.getFlattenedMacrosAndGroups();
		KMEngine.setVariable("msMacrosInfo", JSON.stringify(_result));
		return "OK";
	}

	try {
		return run();
	} catch (e) {
		return "Error: " + e.message;
	}
})()